@@ -207,6 +207,8 @@ class Compiler:
         A finalization method that is called after compilation has finished and
         after all tasks from the finalizers-stack have been executed
         """
+        if isinstance(self.tree, RootNode):
+            self.tree.stage = ''   # turn of stage-verification as default to keep matters simple
         return result
 
     def wildcard(self, node: Node) -> Any:
@@ -248,7 +250,7 @@ class Compiler:
         if self._dirty_flag:
             self.reset()
         self._dirty_flag = True
-        self.tree = cast(RootNode, root) if isinstance(root, RootNode) else RootNode(root)
+        self.tree = root if isinstance(root, RootNode) else RootNode(root)
         self.prepare(self.tree)
         result = self.compile(self.tree)
         while self.finalizers:
@@ -476,6 +478,8 @@ def compile_source(source: str,
                                       AST_TRANSFORM_CRASH)
         else:
             syntax_tree = transformer(syntax_tree)
+        assert isinstance(syntax_tree, RootNode)
+        syntax_tree.stage = 'ast'
 
         if 'ast' in log_syntax_trees:
             log_ST(syntax_tree, log_file_name + '.ast')
@@ -483,6 +487,7 @@ def compile_source(source: str,
         if not is_fatal(syntax_tree.error_flag):
             if preserve_AST:
                 ast = copy.deepcopy(syntax_tree)
+                ast.stage = ''  # turn stage-verification off on copied ast as default
 
             # Compilation
 
@@ -536,7 +541,7 @@ def process_tree(tp: CompilerCallable, tree: RootNode) -> Any:
                 result = tp(tree)
             except Exception as e:
                 # node = tp.path[-1] if hasattr(tp, 'path') and tp.path else tree
-                node = getattr(tp, 'path', [tree])[-1]
+                node = (getattr(tp, 'path', [tree]) or [tree])[-1]
                 st = traceback.format_list(traceback.extract_tb(e.__traceback__))
                 trace = ''.join(st)  # filter_stacktrace(st)
                 tree.new_error(
@@ -558,7 +563,7 @@ def extract_data(tree_or_data: Union[RootNode, Node, Any]) -> Any:
     """Retrieves the data from the given tree or just passes the data through
     if argument ``tree_or_data`` is not of type RootNode."""
     if isinstance(tree_or_data, RootNode):
-        return cast(RootNode, tree_or_data).data
+        return tree_or_data.data
     return tree_or_data
 
 
@@ -576,14 +581,28 @@ def run_pipeline(junctions: Set[Junction],
     essentially a function that transforms a tree from one particular stage
     (identified by its name) to another stage, again itentified by its name.
 
-    TODO: Parallelize processing of junctions
+    TODO: Parallelize processing of junctions? Requires copying a lot ot tree-data!?
     """
     def cmp_junctions(a, b) -> int:
         if a[-1] == b[0]:
-            return -11
+            return -1
         if b[-1] == a[0]:
             return 1
-        return 0
+        if b[-1] > a[-1]:
+            return -1
+        else:
+            return 0
+
+    def verify_stage(given_stage, junction, field, further_info=''):
+        if not given_stage:  return  # verification is considered as turned off
+        assert field in (0, 2)
+        expected_stage = junction[field]
+        stage_type = 'source stage' if field == 0 else 'target stage'
+        if given_stage.lower() != expected_stage.lower():
+            raise AssertionError(f'Expected {stage_type} "{expected_stage}" but found '
+                f'"{given_stage}" in "{junction[0]} -> {junction[2]}"-transformation! '
+                'Possible causes:  a) wrong stage name specified in junction  b) stage name not '
+                f'updated by compilation-function  c) internal error of DHParser. {further_info}')
 
     t_to_j = {j[-1]: j for j in junctions}
     steps = []
@@ -627,9 +646,14 @@ def run_pipeline(junctions: Set[Junction],
                     errata[t] = []
                 else:
                     if not isinstance(tree, RootNode):
-                        raise ValueError(f'Object in stage {s} is not a tree but a {type(tree)} '
-                                         f'and, therefore, cannot be processed to {t}')
+                        raise ValueError(f'Object in stage "{s}" is not a tree but a {type(tree)}'
+                                         f' and, therefore, cannot be processed to {t}')
+                    verify_stage(tree.stage, junction, 0)
                     results[t] = process_tree(junction[1](), tree)
+                    if tree.stage == s:  # tree stage hasn't been set by the processing function
+                        tree.stage = junction[2]
+                    else:
+                        verify_stage(tree.stage, junction, 2)
                     errata[t] = copy.copy(tree.errors_sorted)
     return {t: (extract_data(results[t]), errata[t]) for t in results.keys()}
 
