@@ -115,6 +115,7 @@ __all__ = ('SourceMap',
            'BADLY_NESTED_OPTIONAL_PARSER',
            'BAD_MANDATORY_SETUP',
            'DUPLICATE_PARSERS_IN_ALTERNATIVE',
+           'SYMBOL_NAME_IS_PYTHON_KEYWORD',
            'BAD_ORDER_OF_ALTERNATIVES',
            'BAD_REPETITION_COUNT',
            'MALFORMED_REGULAR_EXPRESSION',
@@ -237,6 +238,7 @@ DIRECTIVE_FOR_NONEXISTANT_SYMBOL         = ErrorCode(1100)
 INAPPROPRIATE_SYMBOL_FOR_DIRECTIVE       = ErrorCode(1110)
 PEG_EXPRESSION_IN_DIRECTIVE_WO_BRACKETS  = ErrorCode(1120)
 CUSTOM_PARSER_FAILURE                    = ErrorCode(1130)
+UNDEFINED_SYMBOL                         = ErrorCode(1140)
 
 ERROR_WHILE_RECOVERING_FROM_ERROR        = ErrorCode(1301)
 
@@ -246,6 +248,7 @@ CAPTURE_WITHOUT_PARSERNAME               = ErrorCode(1510)
 LOOKAHEAD_WITH_OPTIONAL_PARSER           = ErrorCode(1520)
 BADLY_NESTED_OPTIONAL_PARSER             = ErrorCode(1530)
 BAD_MANDATORY_SETUP                      = ErrorCode(1550)
+SYMBOL_NAME_IS_PYTHON_KEYWORD            = ErrorCode(1555)
 DUPLICATE_PARSERS_IN_ALTERNATIVE         = ErrorCode(1560)
 BAD_ORDER_OF_ALTERNATIVES                = ErrorCode(1570)
 BAD_REPETITION_COUNT                     = ErrorCode(1580)
@@ -337,12 +340,22 @@ class Error:
         self.end_column = -1      # type: int
         self.related = tuple(related)   # type: Sequence['Error']
 
+    def _normalize_msg(self, msg: str) -> str:
+        """A hack to support recognition of duplicate error messages when using
+        produced as articaft by the seed and grow algorithm that catches
+        left-recursion in the parser. See :py:meth:`~parse.Forward.__call__`."""
+        if self.code == RESUME_NOTICE:
+            i = msg.find('with ')
+            if i > 0:  msg = msg[:i]
+        return msg
+
     def __eq__(self, other):
-        return self.message == other.message and self.code == other.code \
-               and self._pos == other._pos  # and self.length == other.length
+        return self._normalize_msg(self.message) == self._normalize_msg(other.message) \
+            and self.code == other.code \
+            and self._pos == other._pos  # and self.length == other.length
 
     def __hash__(self):
-        return hash((self.message, self.code, self._pos))
+        return hash((self._normalize_msg(self.message), self.code, self._pos))
 
     def __str__(self):
         if self.orig_doc and self.orig_doc != 'UNKNOWN_FILE':
