@@ -57,7 +57,7 @@ from DHParser.log import is_logging, clear_logs, local_log_dir, log_parsing_hist
 from DHParser.parse import Lookahead
 from DHParser.server import RX_CONTENT_LENGTH, RE_DATA_START, JSONRPC_HEADER_BYTES
 from DHParser.nodetree import Node, RootNode, deserialize, flatten_sxpr, ZOMBIE_TAG
-from DHParser.trace import set_tracer, all_descendants, trace_history
+from DHParser.trace import set_tracer, trace_history
 from DHParser.transform import traverse, remove_children
 from DHParser.toolkit import load_if_file, re, re_find, instantiate_executor, TypeAlias
 
@@ -87,7 +87,7 @@ __all__ = ('unit_from_config',
 UNIT_STAGES = frozenset({'match*', 'match', 'fail', 'ast', 'cst'})
 RESULT_STAGES = frozenset({'__cst__', '__ast__', '__err__'})
 
-RX_SECTION = re.compile(r'\s*\[(?P<stage>\w+):(?P<symbol>\w+)\]')
+RX_SECTION = re.compile(r'\s*\[(?P<stage>\w+(:?\.\w+)*):(?P<symbol>\w+)\]')
 RE_VALUE = '(?:"""((?:.|\n)*?)""")|' + "(?:'''((?:.|\n)*?)''')|" + \
            r'(?:"(.*?)")|' + "(?:'(.*?)')|" + r'(.*(?:\n(?:\s*\n)*    .*)*)'
 # the following does not work with pypy3, because pypy's re-engine does not
@@ -159,7 +159,8 @@ def unit_from_config(config_str, filename, allowed_stages=UNIT_STAGES):
         d = section_match.groupdict()
         stage = d['stage']
         if stage not in allowed_stages:
-            raise KeyError('Unknown stage ' + stage + " ! must be one of: " + str(allowed_stages))
+            raise KeyError(f'Unknown stage "{stage}" in file "{filename}"! '
+                           f'must be one of: {allowed_stages}')
         symbol = d['symbol']
         pos = eat_comments(cfg, section_match.span()[1])
 
@@ -406,7 +407,7 @@ def grammar_unit(test_unit, parser_factory, transformer_factory, report='REPORT'
     def has_lookahead(parser_name: str) -> bool:
         """Returns True if the parser or any of its descendant parsers is a
         Lookahead parser."""
-        return parser[parser_name].apply(lambda ctx: isinstance(ctx[-1], Lookahead))
+        return parser[parser_name].apply(lambda p: isinstance(p, Lookahead))
 
 
     def lookahead_artifact(syntax_tree: Node):
@@ -452,7 +453,7 @@ def grammar_unit(test_unit, parser_factory, transformer_factory, report='REPORT'
         track_history = get_config_value('history_tracking')
         try:
             if has_lookahead(parser_name):
-                set_tracer(all_descendants(parser[parser_name]), trace_history)
+                set_tracer(parser[parser_name].descendants, trace_history)
                 track_history = True
         except AttributeError:
             pass
@@ -672,7 +673,7 @@ def grammar_unit(test_unit, parser_factory, transformer_factory, report='REPORT'
                 write(infostr + ("OK" if len(errata) == errflag else "FAIL"))
 
     # remove tracers, in case there are any:
-    set_tracer(all_descendants(parser.root_parser__), None)
+    set_tracer(parser.root_parser__.descendants, None)
 
     # write test-report
     if report:
