@@ -74,7 +74,7 @@ def preprocessor_factory() -> PreprocessorFunc:
     return chain_preprocessors(include_prep, LaTeXPreprocessor)
 
 
-get_preprocessor = ThreadLocalSingletonFactory(preprocessor_factory, ident="1")
+get_preprocessor = ThreadLocalSingletonFactory(preprocessor_factory)
 
 
 #######################################################################
@@ -92,7 +92,7 @@ class LaTeXGrammar(Grammar):
     paragraph = Forward()
     param_block = Forward()
     tabular_config = Forward()
-    source_hash__ = "53a92f4b161cd96012f0936b1c69ea04"
+    source_hash__ = "e479dcd672c4bb4c2d0fc45f51b099e8"
     disposable__ = re.compile('_\\w+')
     static_analysis_pending__ = []  # type: List[bool]
     parser_initialization__ = ["upon instantiation"]
@@ -156,9 +156,10 @@ class LaTeXGrammar(Grammar):
     sequence = Series(Option(_WSPC), OneOrMore(Series(Alternative(paragraph, _block_environment), Option(Alternative(_PARSEP, S)))))
     block_of_paragraphs = Series(Series(Drop(Text("{")), dwsp__), Option(sequence), Series(Drop(Text("}")), dwsp__), mandatory=2)
     special = Alternative(Drop(Text("\\-")), Series(Drop(RegExp('\\\\')), esc_char), UMLAUT, QUOTEMARK)
-    _structure_name = Drop(Alternative(Drop(Text("subsection")), Drop(Text("section")), Drop(Text("chapter")), Drop(Text("subsubsection")), Drop(Text("paragraph")), Drop(Text("subparagraph")), Drop(Text("item"))))
+    _item_name = Text("item")
+    _structure_name = Drop(Alternative(Drop(Text("subsection")), Drop(Text("section")), Drop(Text("chapter")), Drop(Text("subsubsection")), Drop(Text("paragraph")), Drop(Text("subparagraph"))))
     _env_name = Drop(Alternative(Drop(Text("enumerate")), Drop(Text("itemize")), Drop(Text("description")), Drop(Text("figure")), Drop(Text("quote")), Drop(Text("quotation")), Drop(Text("tabular")), Drop(Series(Drop(Text("displaymath")), Drop(Option(Drop(Text("*")))))), Drop(Series(Drop(Text("equation")), Drop(Option(Drop(Text("*")))))), Drop(Series(Drop(Text("eqnarray")), Drop(Option(Drop(Text("*"))))))))
-    blockcmd = Series(_BACKSLASH, Alternative(Series(Alternative(Series(Drop(Text("begin{")), dwsp__), Series(Drop(Text("end{")), dwsp__)), _env_name, Series(Drop(Text("}")), dwsp__)), Series(_structure_name, Lookahead(Drop(Text("{")))), Drop(Text("[")), Drop(Text("]"))))
+    blockcmd = Series(_BACKSLASH, Alternative(Series(Alternative(Series(Drop(Text("begin{")), dwsp__), Series(Drop(Text("end{")), dwsp__)), _env_name, Series(Drop(Text("}")), dwsp__)), Series(_structure_name, Lookahead(Drop(Text("{")))), Drop(Text("[")), Drop(Text("]")), _item_name))
     no_command = Alternative(Series(Drop(Text("\\begin{")), dwsp__), Series(Drop(Text("\\end{")), dwsp__), Series(_BACKSLASH, _structure_name, Lookahead(Drop(Text("{")))))
     text = Series(OneOrMore(Alternative(_TEXT, special)), ZeroOrMore(Series(S, OneOrMore(Alternative(_TEXT, special)))))
     cfg_text = ZeroOrMore(Alternative(Series(dwsp__, text), CMDNAME, SPECIAL, block))
@@ -417,7 +418,8 @@ LaTeX_AST_transformation_table = {
     "cfg_text": [reduce_single_child],
     "block": [reduce_single_child],
     "flag": [reduce_single_child],
-    "text, urlstring": collapse,
+    "text": collapse,
+    "urlstring": [],
     "no_command, blockcmd": [],
     "_structure_name": [],
     "CMDNAME": [remove_whitespace, reduce_single_child],
@@ -447,7 +449,7 @@ def LaTeXTransformer() -> TransformerCallable:
     threads or processes."""
     return partial(traverse, transformation_table=LaTeX_AST_transformation_table.copy())
 
-get_transformer = ThreadLocalSingletonFactory(LaTeXTransformer, ident="1")
+get_transformer = ThreadLocalSingletonFactory(LaTeXTransformer)
 
 def transform_LaTeX(cst):
     get_transformer()(cst)
@@ -825,7 +827,7 @@ class LaTeXCompiler(Compiler):
     #     return node
 
 
-get_compiler = ThreadLocalSingletonFactory(LaTeXCompiler, ident="1")
+get_compiler = ThreadLocalSingletonFactory(LaTeXCompiler)
 
 def compile_LaTeX(ast):
     return get_compiler()(ast)
