@@ -37,7 +37,7 @@ from DHParser.error import has_errors, MANDATORY_CONTINUATION, PARSER_STOPPED_BE
     REDEFINED_DIRECTIVE, UNUSED_ERROR_HANDLING_WARNING, AMBIGUOUS_ERROR_HANDLING, \
     REORDERING_OF_ALTERNATIVES_REQUIRED, BAD_ORDER_OF_ALTERNATIVES, UNCONNECTED_SYMBOL_WARNING, \
     PEG_EXPRESSION_IN_DIRECTIVE_WO_BRACKETS, ERROR, WARNING, \
-    ZERO_LENGTH_CAPTURE_POSSIBLE_WARNING, canonical_error_strings
+    ZERO_LENGTH_CAPTURE_POSSIBLE_WARNING, SYMBOL_NAME_IS_PYTHON_KEYWORD, canonical_error_strings
 from DHParser.nodetree import WHITESPACE_PTYPE, flatten_sxpr
 from DHParser.ebnf import get_ebnf_grammar, get_ebnf_transformer, EBNFTransform, \
     EBNFDirectives, get_ebnf_compiler, compile_ebnf, DHPARSER_IMPORTS, parse_ebnf, \
@@ -286,6 +286,18 @@ class TestParserNameOverwriteBug:
         assert not st.error_flag
 
 
+class TestEBNFErrors:
+    def test_Python_keywords(self):
+        ebnf = '''and = /.*/\n'''
+        try:
+            parser = create_parser(ebnf)
+            assert False, "CompilationError expected"
+        except SyntaxError:
+            assert False, "SyntaxError should have been avoided."
+        except CompilationError as ce:
+            assert any(err.code == SYMBOL_NAME_IS_PYTHON_KEYWORD
+                       for err in ce.errors)
+
 class TestSemanticValidation:
     def check(self, minilang, bool_filter=lambda x: x):
         grammar = get_ebnf_grammar()
@@ -806,13 +818,20 @@ class TestErrorCustomizationErrors:
         l3 = [zeile for zeile in l2 if not zeile.lstrip().startswith('mitte')]
         lang3 = '\n'.join(l3).replace('mitte', '(`M` §"ITTE")')
         result, messages, ast = compile_ebnf(lang3)
+        # print(lang3)
+        # print(result)
         assert not messages, str(messages)
 
         parser = create_parser(lang3)
         cst = parser('ANFANGMITTEENDE')
         assert not cst.errors
+        # parser.root_parser__.apply(lambda p: p.reset())
+        # for p in parser.all_parsers__:
+        #     if len(p.visited) > 0:
+        #         print(p.name, p)
         cst = parser('ANFANGMISSEENDE')
         assert cst.errors
+        # print(cst.as_sxpr())
         assert 'alles' in cst and 'ZOMBIE__' in cst['alles'] and 'ende' in cst['alles']
 
     def test_multiple_resume_definitions(self):
@@ -859,6 +878,30 @@ class TestErrorCustomizationErrors:
         result = parser('ADX')
         assert "several strings" in str(result.errors), str(result.errors)
 
+    # def test_indirect_error_handling(self):
+    #     lang = """
+    #         document = series
+    #         @series_error = 'error'
+    #         series = "A" B "C"
+    #         B = "B" § "b"
+    #         """
+    #     provider = grammar_provider(lang)
+    #     parser = provider()
+    #     result = parser('ABC')
+    #     print(result.errors)
+    #     # result, messages, ast = compile_ebnf(lang)
+    #     # assert not messages, "No warning expected, but: " + str(messages)
+    #
+    # def test_indirect_ambiguity(self):
+    #     lang = """
+    #         document = series
+    #         @series_error = 'error'
+    #         series = "A" B § "C"
+    #         B = "B" § "b"
+    #         """
+    #     result, messages, ast = compile_ebnf(lang)
+    #     assert messages, '"Ambigous error message"-warning expected!'
+
 
 class TestVariableCapture:
     tree_grammar = '''@whitespace = horizontal
@@ -1031,6 +1074,7 @@ class TestCustomizedResumeParsing_with_Parsers:
         # because of resuming, there should be only on error message
         assert len(cst.errors_sorted) == 1
 
+
 class TestCustomizedResumeParsing_with_autogenerated_Parsers:
     lang = """@ literalws   = right
     @ disposable  = /_\w+/
@@ -1864,7 +1908,7 @@ class TestErrors:
     numbers = r"""@whitespace = vertical
     document = { ~ number | error } ~ EOF
     number = /[1-9]\d*/~
-    error  = !EOF @Error("Not a valid Number!") /.*?(?=\s|$)/  # skip to next whitespace
+    error  = !EOF @Error("»{1}« is not a valid Number!") /.*?(?=\s|$)/  # skip to next whitespace
     EOF = !/./
     """
 
@@ -1879,19 +1923,38 @@ class TestErrors:
     (:RegExp "2")
     (:Whitespace " "))
   (error
-    (ZOMBIE__ `(err "1:3: Error (1000): Not a valid Number!"))
+    (ZOMBIE__ `(err "1:3: Error (1000): »X 3 Y 4...« is not a valid Number!"))
     (:RegExp "X"))
   (:Whitespace " ")
   (number
     (:RegExp "3")
     (:Whitespace " "))
   (error
-    (ZOMBIE__ `(err "1:7: Error (1000): Not a valid Number!"))
+    (ZOMBIE__ `(err "1:7: Error (1000): »Y 4...« is not a valid Number!"))
     (:RegExp "Y"))
   (:Whitespace " ")
   (number "4")
   (EOF))"""
 
+    def test_missing_at(self):
+        try:
+            parser = create_parser(self.numbers.replace('@Error', 'Error'))
+            assert False, "CompilationError expected"
+        except CompilationError as ce:
+            assert str(ce).find('"@" should be added')
+
+
+class TestNameConflicts:
+    def test_symbol_shadows_parser_name(self):
+        gr = '''
+        doc = Text | Literal
+        Literal = `abc` 
+        Text = /\w+/
+        '''
+        # should run without causig an exception
+        p = create_parser(gr)
+        assert p.python_src__.find('parse_namespace__.Text("abc")') >= 0
+
 
 if __name__ == "__main__":
     from DHParser.testing import runner
